@@ -1,7 +1,12 @@
 #!/usr/bin/env python
 #
-# $Id: _psbsd.py 944 2011-03-04 17:05:24Z g.rodola $
+# $Id: _psbsd.py 1206 2011-10-26 21:13:51Z g.rodola $
 #
+# Copyright (c) 2009, Jay Loden, Giampaolo Rodola'. All rights reserved.
+# Use of this source code is governed by a BSD-style license that can be
+# found in the LICENSE file.
+
+"""FreeBSD platform implementation."""
 
 import errno
 import os
@@ -13,49 +18,85 @@ from psutil.error import AccessDenied, NoSuchProcess, TimeoutExpired
 from psutil._compat import namedtuple
 from psutil._common import *
 
-__all__ = base_module_namespace[:]
+__extra__all__ = []
 
 # --- constants
 
 NUM_CPUS = _psutil_bsd.get_num_cpus()
-TOTAL_PHYMEM = _psutil_bsd.get_total_phymem()
 BOOT_TIME = _psutil_bsd.get_system_boot_time()
+_TERMINAL_MAP = _psposix._get_terminal_map()
+_cputimes_ntuple = namedtuple('cputimes', 'user nice system idle irq')
 
 # --- public functions
 
-def avail_phymem():
-    "Return the amount of physical memory available on the system, in bytes."
-    return _psutil_bsd.get_avail_phymem()
+def phymem_usage():
+    """Physical system memory as a (total, used, free) tuple."""
+    total = _psutil_bsd.get_total_phymem()
+    free =  _psutil_bsd.get_avail_phymem()
+    used = total - free
+    # XXX check out whether we have to do the same math we do on Linux
+    percent = usage_percent(used, total, _round=1)
+    return ntuple_sysmeminfo(total, used, free, percent)
 
-def used_phymem():
-    "Return the amount of physical memory currently in use on the system, in bytes."
-    return TOTAL_PHYMEM - _psutil_bsd.get_avail_phymem()
+def virtmem_usage():
+    """Virtual system memory as a (total, used, free) tuple."""
+    total = _psutil_bsd.get_total_virtmem()
+    free =  _psutil_bsd.get_avail_virtmem()
+    used = total - free
+    percent = usage_percent(used, total, _round=1)
+    return ntuple_sysmeminfo(total, used, free, percent)
 
-def total_virtmem():
-    "Return the amount of total virtual memory available on the system, in bytes."
-    return _psutil_bsd.get_total_virtmem()
-
-def avail_virtmem():
-    "Return the amount of virtual memory currently in use on the system, in bytes."
-    return _psutil_bsd.get_avail_virtmem()
-
-def used_virtmem():
-    """Return the amount of used memory currently in use on the system, in bytes."""
-    return _psutil_bsd.get_total_virtmem() - _psutil_bsd.get_avail_virtmem()
-
-_cputimes_ntuple = namedtuple('cputimes', 'user nice system idle irq')
 def get_system_cpu_times():
-    """Return system CPU times as a named tuple"""
+    """Return system per-CPU times as a named tuple"""
     user, nice, system, idle, irq = _psutil_bsd.get_system_cpu_times()
     return _cputimes_ntuple(user, nice, system, idle, irq)
 
-def get_pid_list():
-    """Returns a list of PIDs currently running on the system."""
-    return _psutil_bsd.get_pid_list()
+def get_system_per_cpu_times():
+    """Return system CPU times as a named tuple"""
+    ret = []
+    for cpu_t in _psutil_bsd.get_system_per_cpu_times():
+        user, nice, system, idle, irq = cpu_t
+        item = _cputimes_ntuple(user, nice, system, idle, irq)
+        ret.append(item)
+    return ret
+    
+# XXX
+# Ok, this is very dirty.
+# On FreeBSD < 8 we cannot gather per-cpu information, see:
+# http://code.google.com/p/psutil/issues/detail?id=226 
+# If NUM_CPUS > 1, on first call we return single cpu times to avoid a 
+# crash at psutil import time.
+# Next calls will fail with NotImplementedError
+if not hasattr(_psutil_bsd, "get_system_per_cpu_times"):
+    def get_system_per_cpu_times():
+        if NUM_CPUS == 1:
+            return [get_system_cpu_times]
+        if get_system_per_cpu_times.__called__:
+            raise NotImplementedError("supported only starting from FreeBSD 8")
+        get_system_per_cpu_times.__called__ = True
+        return [get_system_cpu_times]
+get_system_per_cpu_times.__called__ = False
 
-def pid_exists(pid):
-    """Check For the existence of a unix pid."""
-    return _psposix.pid_exists(pid)
+def disk_partitions(all=False):
+    retlist = []
+    partitions = _psutil_bsd.get_disk_partitions()
+    for partition in partitions:
+        device, mountpoint, fstype = partition
+        if device == 'none':
+            device = ''
+        if not all:
+            if not os.path.isabs(device) \
+            or not os.path.exists(device):
+                continue
+        ntuple = ntuple_partition(device, mountpoint, fstype)
+        retlist.append(ntuple)
+    return retlist
+
+get_pid_list = _psutil_bsd.get_pid_list
+pid_exists = _psposix.pid_exists
+get_disk_usage = _psposix.get_disk_usage
+network_io_counters = _psutil_bsd.get_network_io_counters
+disk_io_counters = _psutil_bsd.get_disk_io_counters
 
 
 def wrap_exceptions(method):
@@ -85,7 +126,7 @@ _status_map = {
 }
 
 
-class BSDProcess(object):
+class Process(object):
     """Wrapper class around underlying C implementation."""
 
     __slots__ = ["pid", "_process_name"]
@@ -109,11 +150,26 @@ class BSDProcess(object):
         """Return process cmdline as a list of arguments."""
         return _psutil_bsd.get_process_cmdline(self.pid)
 
+    @wrap_exceptions
+    def get_process_terminal(self):
+        tty_nr = _psutil_bsd.get_process_tty_nr(self.pid)
+        try:
+            return _TERMINAL_MAP[tty_nr]
+        except KeyError:
+            return None
+
     @wrap_exceptions
     def get_process_ppid(self):
         """Return process parent pid."""
         return _psutil_bsd.get_process_ppid(self.pid)
 
+    # XXX - available on FreeBSD >= 8 only
+    if hasattr(_psutil_bsd, "get_process_cwd"):
+        @wrap_exceptions
+        def get_process_cwd(self):
+            """Return process current working directory."""
+            return _psutil_bsd.get_process_cwd(self.pid)
+
     @wrap_exceptions
     def get_process_uids(self):
         """Return real, effective and saved user ids."""
@@ -159,18 +215,32 @@ class BSDProcess(object):
             retlist.append(ntuple)
         return retlist
 
-
+    @wrap_exceptions
     def get_open_files(self):
-        """Return files opened by process by parsing lsof output."""
-        lsof = _psposix.LsofParser(self.pid, self._process_name)
-        return lsof.get_process_open_files()
+        """Return files opened by process as a list of namedtuples."""
+        # XXX - C implementation available on FreeBSD >= 8 only
+        # else fallback on lsof parser
+        if hasattr(_psutil_bsd, "get_process_open_files"):
+            rawlist = _psutil_bsd.get_process_open_files(self.pid)
+            return [ntuple_openfile(path, fd) for path, fd in rawlist]
+        else:
+            lsof = _psposix.LsofParser(self.pid, self._process_name)            
+            return lsof.get_process_open_files()
 
-    def get_connections(self):
+    def get_connections(self, kind='inet'):
         """Return network connections opened by a process as a list of
         namedtuples by parsing lsof output.
         """
+        if kind not in conn_tmap:
+            raise ValueError("invalid %r kind argument; choose between %s"
+                             % (kind, ', '.join([repr(x) for x in conn_tmap])))
+        families, types = conn_tmap[kind]
+        ret = []
         lsof = _psposix.LsofParser(self.pid, self._process_name)
-        return lsof.get_process_connections()
+        for conn in lsof.get_process_connections():
+            if conn.family in families and conn.type in types:
+                ret.append(conn)
+        return ret
 
     @wrap_exceptions
     def process_wait(self, timeout=None):
@@ -198,7 +268,3 @@ class BSDProcess(object):
     def get_process_io_counters(self):
         rc, wc, rb, wb = _psutil_bsd.get_process_io_counters(self.pid)
         return ntuple_io(rc, wc, rb, wb)
-
-
-PlatformProcess = BSDProcess
-
