@@ -1,7 +1,12 @@
 #!/usr/bin/env python
 #
-# $Id: _pslinux.py 953 2011-03-20 20:29:45Z g.rodola $
+# $Id: _pslinux.py 1173 2011-10-19 19:29:46Z g.rodola $
 #
+# Copyright (c) 2009, Jay Loden, Giampaolo Rodola'. All rights reserved.
+# Use of this source code is governed by a BSD-style license that can be
+# found in the LICENSE file.
+
+"""Linux platform implementation."""
 
 import os
 import errno
@@ -9,62 +14,81 @@ import socket
 import struct
 import sys
 import base64
+import re
 
 import _psutil_posix
 import _psutil_linux
 from psutil import _psposix
 from psutil.error import AccessDenied, NoSuchProcess, TimeoutExpired
-from psutil._compat import namedtuple
 from psutil._common import *
 
-# Linux specific extended namespace
-__all__ = base_module_namespace[:]
-__all__.extend([
-    "IOPRIO_CLASS_NONE", "IOPRIO_CLASS_RT", "IOPRIO_CLASS_BE", "IOPRIO_CLASS_IDLE",
-    "phymem_buffers", "cached_phymem",
-    ])
+__extra__all__ = [
+    "IOPRIO_CLASS_NONE", "IOPRIO_CLASS_RT", "IOPRIO_CLASS_BE",
+    "IOPRIO_CLASS_IDLE",
+    "phymem_buffers", "cached_phymem"]
 
 
 def _get_boot_time():
     """Return system boot time (epoch in seconds)"""
     f = open('/proc/stat', 'r')
-    for line in f:
-        if line.startswith('btime'):
-            f.close()
-            return float(line.strip().split()[1])
+    try:
+        for line in f:
+            if line.startswith('btime'):
+                return float(line.strip().split()[1])
+        raise RuntimeError("line not found")
+    finally:
+        f.close()
 
 def _get_num_cpus():
     """Return the number of CPUs on the system"""
-    num = 0
-    f = open('/proc/cpuinfo', 'r')
-    for line in f:
-        if line.startswith('processor'):
-            num += 1
-    f.close()
-    return num
-
-def _get_total_phymem():
-    """Return the total amount of physical memory, in bytes"""
-    f = open('/proc/meminfo', 'r')
-    for line in f:
-        if line.startswith('MemTotal:'):
+    # we try to determine num CPUs by using different approaches.
+    # SC_NPROCESSORS_ONLN seems to be the safer and it is also
+    # used by multiprocessing module
+    try:
+        return os.sysconf("SC_NPROCESSORS_ONLN")
+    except ValueError:
+        # as a second fallback we try to parse /proc/cpuinfo
+        num = 0
+        f = open('/proc/cpuinfo', 'r')
+        try:
+            lines = f.readlines()
+        finally:
             f.close()
-            return int(line.split()[1]) * 1024
+        for line in lines:
+            if line.lower().startswith('processor'):
+                num += 1
+
+    # unknown format (e.g. amrel/sparc architectures), see:
+    # http://code.google.com/p/psutil/issues/detail?id=200
+    # try to parse /proc/stat as a last resort
+    if num == 0:
+        f = open('/proc/stat', 'r')
+        try:
+            lines = f.readlines()
+        finally:
+            f.close()
+        search = re.compile('cpu\d')
+        for line in lines:
+            line = line.split(' ')[0]
+            if search.match(line):
+                num += 1
+
+    if num == 0:
+        raise RuntimeError("can't determine number of CPUs")
+    return num
 
 
 # Number of clock ticks per second
 _CLOCK_TICKS = os.sysconf(os.sysconf_names["SC_CLK_TCK"])
+_TERMINAL_MAP = _psposix._get_terminal_map()
 BOOT_TIME = _get_boot_time()
 NUM_CPUS = _get_num_cpus()
-TOTAL_PHYMEM = _get_total_phymem()
 # ioprio_* constants http://linux.die.net/man/2/ioprio_get
 IOPRIO_CLASS_NONE = 0
 IOPRIO_CLASS_RT = 1
 IOPRIO_CLASS_BE = 2
 IOPRIO_CLASS_IDLE = 3
 
-del _get_boot_time, _get_num_cpus, _get_total_phymem
-
 # http://students.mimuw.edu.pl/lxr/source/include/net/tcp_states.h
 _TCP_STATES_TABLE = {"01" : "ESTABLISHED",
                      "02" : "SYN_SENT",
@@ -79,90 +103,199 @@ _TCP_STATES_TABLE = {"01" : "ESTABLISHED",
                      "0B" : "CLOSING"
                      }
 
-def avail_phymem():
-    """Return the amount of physical memory available, in bytes."""
-    f = open('/proc/meminfo', 'r')
-    free = None
-    _flag = False
-    for line in f:
-        if line.startswith('MemFree:'):
-            free = int(line.split()[1]) * 1024
-            break
-    f.close()
-    return free
-
-def used_phymem():
-    """"Return the amount of physical memory used, in bytes."""
-    return (TOTAL_PHYMEM - avail_phymem())
-
-def total_virtmem():
-    """"Return the total amount of virtual memory, in bytes."""
-    f = open('/proc/meminfo', 'r')
-    for line in f:
-        if line.startswith('SwapTotal:'):
-            f.close()
-            return int(line.split()[1]) * 1024
-
-def avail_virtmem():
-    """Return the amount of virtual memory currently in use on the
-    system, in bytes.
-    """
-    f = open('/proc/meminfo', 'r')
-    for line in f:
-        if line.startswith('SwapFree:'):
-            f.close()
-            return int(line.split()[1]) * 1024
-
-def used_virtmem():
-    """Return the amount of used memory currently in use on the system,
-    in bytes.
-    """
-    return total_virtmem() - avail_virtmem()
+# --- system memory functions
 
 def cached_phymem():
     """Return the amount of cached memory on the system, in bytes.
     This reflects the "cached" column of free command line utility.
     """
     f = open('/proc/meminfo', 'r')
-    for line in f:
-        if line.startswith('Cached:'):
-            f.close()
-            return int(line.split()[1]) * 1024
+    try:
+        for line in f:
+            if line.startswith('Cached:'):
+                return int(line.split()[1]) * 1024
+        raise RuntimeError("line not found")
+    finally:
+        f.close()
 
 def phymem_buffers():
     """Return the amount of physical memory buffers used by the
     kernel in bytes.
     This reflects the "buffers" column of free command line utility.
     """
+    total, free, buffers = _psutil_linux.get_physmem()
+    return buffers
+
+def phymem_usage():
+    """Return physical memory usage statistics as a namedutple including
+    tota, used, free and percent usage.
+    """
+    # total, used and free values are matched against free cmdline utility
+    # the percentage matches top/htop and gnome-system-monitor
+    total, free, buffers = _psutil_linux.get_physmem()
+    cached = cached_phymem()
+    used = total - free
+    percent = usage_percent(total - (free + buffers + cached), total, _round=1)
+    return ntuple_sysmeminfo(total, used, free, percent)
+
+
+def virtmem_usage():
     f = open('/proc/meminfo', 'r')
-    for line in f:
-        if line.startswith('Buffers:'):
-            f.close()
-            return int(line.split()[1]) * 1024
+    try:
+        total = free = None
+        for line in f:
+            if line.startswith('SwapTotal:'):
+                total = int(line.split()[1]) * 1024
+            elif line.startswith('SwapFree:'):
+                free = int(line.split()[1]) * 1024
+            if total is not None and free is not None:
+                break
+        assert total is not None and free is not None
+        used = total - free
+        percent = usage_percent(used, total, _round=1)
+        return ntuple_sysmeminfo(total, used, free, percent)
+    finally:
+        f.close()
+
+
+# --- system CPU functions
 
 def get_system_cpu_times():
-    """Return a dict representing the following CPU times:
+    """Return a named tuple representing the following CPU times:
     user, nice, system, idle, iowait, irq, softirq.
     """
     f = open('/proc/stat', 'r')
-    values = f.readline().split()
-    f.close()
+    try:
+        values = f.readline().split()
+    finally:
+        f.close()
 
     values = values[1:8]
     values = tuple([float(x) / _CLOCK_TICKS for x in values])
     return ntuple_sys_cputimes(*values[:7])
 
+def get_system_per_cpu_times():
+    """Return a list of namedtuple representing the CPU times
+    for every CPU available on the system.
+    """
+    cpus = []
+    f = open('/proc/stat', 'r')
+    # get rid of the first line who refers to system wide CPU stats
+    try:
+        f.readline()
+        for line in f.readlines():
+            if line.startswith('cpu'):
+                values = line.split()[1:8]
+                values = tuple([float(x) / _CLOCK_TICKS for x in values])
+                entry = ntuple_sys_cputimes(*values[:7])
+                cpus.append(entry)
+        return cpus
+    finally:
+        f.close()
+
+
+# --- system disk functions
+
+def disk_partitions(all=False):
+    """Return mounted disk partitions as a list of nameduples"""
+    phydevs = []
+    f = open("/proc/filesystems", "r")
+    try:
+        for line in f:
+            if not line.startswith("nodev"):
+                phydevs.append(line.strip())
+    finally:
+        f.close()
+
+    retlist = []
+    partitions = _psutil_linux.get_disk_partitions()
+    for partition in partitions:
+        device, mountpoint, fstype = partition
+        if device == 'none':
+            device = ''
+        if not all:
+            if device == '' or fstype not in phydevs:
+                continue
+        ntuple = ntuple_partition(device, mountpoint, fstype)
+        retlist.append(ntuple)
+    return retlist
+
+get_disk_usage = _psposix.get_disk_usage
+
+# --- process functions
+
 def get_pid_list():
     """Returns a list of PIDs currently running on the system."""
     pids = [int(x) for x in os.listdir('/proc') if x.isdigit()]
-    # special case for 0 (kernel process) PID
-    pids.insert(0, 0)
     return pids
 
 def pid_exists(pid):
     """Check For the existence of a unix pid."""
     return _psposix.pid_exists(pid)
 
+def network_io_counters():
+    """Return network I/O statistics for every network interface
+    installed on the system as a dict of raw tuples.
+    """
+    f = open("/proc/net/dev", "r")
+    try:
+        lines = f.readlines()
+    finally:
+        f.close()
+
+    retdict = {}
+    for line in lines[2:]:
+        fields = line.split()
+        name = fields[0][:-1]
+        bytes_recv = int(fields[1])
+        packets_recv = int(fields[2])
+        bytes_sent = int(fields[9])
+        packets_sent = int(fields[10])
+        retdict[name] = (bytes_sent, bytes_recv, packets_sent, packets_recv)
+    return retdict
+
+def disk_io_counters():
+    """Return disk I/O statistics for every disk installed on the
+    system as a dict of raw tuples.
+    """
+    # man iostat states that sectors are equivalent with blocks and
+    # have a size of 512 bytes since 2.4 kernels. This value is
+    # needed to calculate the amount of disk I/O in bytes.
+    SECTOR_SIZE = 512
+
+    # determine partitions we want to look for
+    partitions = []
+    f = open("/proc/partitions", "r")
+    try:
+        lines = f.readlines()[2:]
+    finally:
+        f.close()
+    for line in lines:
+        _, _, _, name = line.split()
+        if name[-1].isdigit():
+            partitions.append(name)
+    #
+    retdict = {}
+    f = open("/proc/diskstats", "r")
+    try:
+        lines = f.readlines()
+    finally:
+        f.close()
+    for line in lines:
+        _, _, name, reads, _, rbytes, rtime, writes, _, wbytes, wtime = \
+            line.split()[:11]
+        if name in partitions:
+            rbytes = int(rbytes) * SECTOR_SIZE
+            wbytes = int(wbytes) * SECTOR_SIZE
+            reads = int(reads)
+            writes = int(writes)
+            # TODO: times are expressed in milliseconds while OSX/BSD has
+            # these expressed in nanoseconds; figure this out.
+            rtime = int(rtime)
+            wtime = int(wtime)
+            retdict[name] = (reads, writes, rbytes, wbytes, rtime, wtime)
+    return retdict
+
 
 # taken from /fs/proc/array.c
 _status_map = {"R" : STATUS_RUNNING,
@@ -197,7 +330,7 @@ def wrap_exceptions(callable):
     return wrapper
 
 
-class LinuxProcess(object):
+class Process(object):
     """Linux process implementation."""
 
     __slots__ = ["pid", "_process_name"]
@@ -208,8 +341,6 @@ class LinuxProcess(object):
 
     @wrap_exceptions
     def get_process_name(self):
-        if self.pid == 0:
-            return 'sched'  # special case for kernel process
         f = open("/proc/%s/stat" % self.pid)
         try:
             name = f.read().split(' ')[1].replace('(', '').replace(')', '')
@@ -252,8 +383,6 @@ class LinuxProcess(object):
 
     @wrap_exceptions
     def get_process_cmdline(self):
-        if self.pid == 0:
-            return []   # special case for kernel process
         f = open("/proc/%s/cmdline" % self.pid)
         try:
             # return the args as a list
@@ -261,31 +390,42 @@ class LinuxProcess(object):
         finally:
             f.close()
 
+    @wrap_exceptions
+    def get_process_terminal(self):
+        f = open("/proc/%s/stat" % self.pid)
+        try:
+            tty_nr = int(f.read().split(' ')[6])
+        finally:
+            f.close()
+        try:
+            return _TERMINAL_MAP[tty_nr]
+        except KeyError:
+            return None
+
     @wrap_exceptions
     def get_process_io_counters(self):
-        # special case for 0 (kernel process) PID
-        if self.pid == 0:
-            return ntuple_io(0, 0, 0, 0)
         f = open("/proc/%s/io" % self.pid)
-        for line in f:
-            if line.startswith("rchar"):
-                read_count = int(line.split()[1])
-            elif line.startswith("wchar"):
-                write_count = int(line.split()[1])
-            elif line.startswith("read_bytes"):
-                read_bytes = int(line.split()[1])
-            elif line.startswith("write_bytes"):
-                write_bytes = int(line.split()[1])
-        return ntuple_io(read_count, write_count, read_bytes, write_bytes)
+        try:
+            for line in f:
+                if line.startswith("rchar"):
+                    read_count = int(line.split()[1])
+                elif line.startswith("wchar"):
+                    write_count = int(line.split()[1])
+                elif line.startswith("read_bytes"):
+                    read_bytes = int(line.split()[1])
+                elif line.startswith("write_bytes"):
+                    write_bytes = int(line.split()[1])
+            return ntuple_io(read_count, write_count, read_bytes, write_bytes)
+        finally:
+            f.close()
 
     @wrap_exceptions
     def get_cpu_times(self):
-        # special case for 0 (kernel process) PID
-        if self.pid == 0:
-            return ntuple_cputimes(0.0, 0.0)
         f = open("/proc/%s/stat" % self.pid)
-        st = f.read().strip()
-        f.close()
+        try:
+            st = f.read().strip()
+        finally:
+            f.close()
         # ignore the first two values ("pid (exe)")
         st = st[st.find(')') + 2:]
         values = st.split(' ')
@@ -302,12 +442,11 @@ class LinuxProcess(object):
 
     @wrap_exceptions
     def get_process_create_time(self):
-        # special case for 0 (kernel processes) PID; return system uptime
-        if self.pid == 0:
-            return BOOT_TIME
         f = open("/proc/%s/stat" % self.pid)
-        st = f.read().strip()
-        f.close()
+        try:
+            st = f.read().strip()
+        finally:
+            f.close()
         # ignore the first two values ("pid (exe)")
         st = st[st.find(')') + 2:]
         values = st.split(' ')
@@ -320,27 +459,24 @@ class LinuxProcess(object):
 
     @wrap_exceptions
     def get_memory_info(self):
-        # special case for 0 (kernel processes) PID
-        if self.pid == 0:
-            return ntuple_meminfo(0, 0)
         f = open("/proc/%s/status" % self.pid)
-        virtual_size = 0
-        resident_size = 0
-        _flag = False
-        for line in f:
-            if (not _flag) and line.startswith("VmSize:"):
-                virtual_size = int(line.split()[1]) * 1024
-                _flag = True
-            elif line.startswith("VmRSS"):
-                resident_size = int(line.split()[1]) * 1024
-                break
-        f.close()
-        return ntuple_meminfo(resident_size, virtual_size)
+        try:
+            virtual_size = 0
+            resident_size = 0
+            _flag = False
+            for line in f:
+                if (not _flag) and line.startswith("VmSize:"):
+                    virtual_size = int(line.split()[1]) * 1024
+                    _flag = True
+                elif line.startswith("VmRSS"):
+                    resident_size = int(line.split()[1]) * 1024
+                    break
+            return ntuple_meminfo(resident_size, virtual_size)
+        finally:
+            f.close()
 
     @wrap_exceptions
     def get_process_cwd(self):
-        if self.pid == 0:
-            raise AccessDenied(self.pid, self._process_name)
         # readlink() might return paths containing null bytes causing
         # problems when used with other fs-related functions (os.*,
         # open(), ...)
@@ -349,18 +485,17 @@ class LinuxProcess(object):
 
     @wrap_exceptions
     def get_process_num_threads(self):
-        if self.pid == 0:
-            return 0
         f = open("/proc/%s/status" % self.pid)
-        for line in f:
-            if line.startswith("Threads:"):
-                f.close()
-                return int(line.split()[1])
+        try:
+            for line in f:
+                if line.startswith("Threads:"):
+                    return int(line.split()[1])
+            raise RuntimeError("line not found")
+        finally:
+            f.close()
 
     @wrap_exceptions
     def get_process_threads(self):
-        if self.pid == 0:
-            return []
         thread_ids = os.listdir("/proc/%s/task" % self.pid)
         thread_ids.sort()
         retlist = []
@@ -373,8 +508,10 @@ class LinuxProcess(object):
                     # disappeared on us
                     continue
                 raise
-            st = f.read().strip()
-            f.close()
+            try:
+                st = f.read().strip()
+            finally:
+                f.close()
             # ignore the first two values ("pid (exe)")
             st = st[st.find(')') + 2:]
             values = st.split(' ')
@@ -387,9 +524,11 @@ class LinuxProcess(object):
     @wrap_exceptions
     def get_process_nice(self):
         #f = open('/proc/%s/stat' % self.pid, 'r')
-        #data = f.read()
-        #f.close()
-        #return int(data.split()[18])
+        #try:
+        #   data = f.read()
+        #   return int(data.split()[18])
+        #finally:
+        #   f.close()
 
         # Use C implementation
         return _psutil_posix.getpriority(self.pid)
@@ -428,16 +567,16 @@ class LinuxProcess(object):
 
     @wrap_exceptions
     def get_process_status(self):
-        if self.pid == 0:
-            return 0
         f = open("/proc/%s/status" % self.pid)
-        for line in f:
-            if line.startswith("State:"):
-                f.close()
-                letter = line.split()[1]
-                if letter in _status_map:
-                    return _status_map[letter]
-                return constant(-1, '?')
+        try:
+            for line in f:
+                if line.startswith("State:"):
+                    letter = line.split()[1]
+                    if letter in _status_map:
+                        return _status_map[letter]
+                    return constant(-1, '?')
+        finally:
+            f.close()
 
     @wrap_exceptions
     def get_open_files(self):
@@ -459,9 +598,23 @@ class LinuxProcess(object):
         return retlist
 
     @wrap_exceptions
-    def get_connections(self):
-        if self.pid == 0:
-            return []
+    def get_connections(self, kind='inet'):
+        """Return connections opened by process as a list of namedtuples.
+        The kind parameter filters for connections that fit the following
+        criteria:
+
+        Kind Value      Number of connections using
+        inet            IPv4 and IPv6
+        inet4           IPv4
+        inet6           IPv6
+        tcp             TCP
+        tcp4            TCP over IPv4
+        tcp6            TCP over IPv6
+        udp             UDP
+        udp4            UDP over IPv4
+        udp6            UDP over IPv6
+        all             the sum of all the possible families and protocols
+        """
         inodes = {}
         # os.listdir() is gonna raise a lot of access denied
         # exceptions in case of unprivileged user; that's fine:
@@ -480,31 +633,65 @@ class LinuxProcess(object):
             # no connections for this process
             return []
 
-        def process(file, family, _type):
+        def process(file, family, type_):
             retlist = []
-            f = open(file)
-            f.readline()  # skip the first line
-            for line in f:
-                _, laddr, raddr, status, _, _, _, _, _, inode = line.split()[:10]
-                if inode in inodes:
-                    laddr = self._decode_address(laddr, family)
-                    raddr = self._decode_address(raddr, family)
-                    if _type == socket.SOCK_STREAM:
-                        status = _TCP_STATES_TABLE[status]
+            try:
+                f = open(file, 'r')
+            except IOError, err:
+                # IPv6 not supported on this platform
+                if err.errno == errno.ENOENT and file.endswith('6'):
+                    return []
+                else:
+                    raise
+            try:
+                f.readline()  # skip the first line
+                for line in f:
+                    # IPv4 / IPv6
+                    if family in (socket.AF_INET, socket.AF_INET6):
+                        _, laddr, raddr, status, _, _, _, _, _, inode = \
+                                                                line.split()[:10]
+                        if inode in inodes:
+                            laddr = self._decode_address(laddr, family)
+                            raddr = self._decode_address(raddr, family)
+                            if type_ == socket.SOCK_STREAM:
+                                status = _TCP_STATES_TABLE[status]
+                            else:
+                                status = ""
+                            fd = int(inodes[inode])
+                            conn = ntuple_connection(fd, family, type_, laddr,
+                                                     raddr, status)
+                            retlist.append(conn)
                     else:
-                        status = ""
-                    fd = int(inodes[inode])
-                    conn = ntuple_connection(fd, family, _type, laddr,
-                                             raddr, status)
-                    retlist.append(conn)
-            f.close()
-            return retlist
+                        raise ValueError(family)
+                return retlist
+            finally:
+                f.close()
+
+        tcp4 = ("tcp" , socket.AF_INET , socket.SOCK_STREAM)
+        tcp6 = ("tcp6", socket.AF_INET6, socket.SOCK_STREAM)
+        udp4 = ("udp" , socket.AF_INET , socket.SOCK_DGRAM)
+        udp6 = ("udp6", socket.AF_INET6, socket.SOCK_DGRAM)
+
+        tmap = {
+            "all"  : (tcp4, tcp6, udp4, udp6),
+            "tcp"  : (tcp4, tcp6),
+            "tcp4" : (tcp4,),
+            "tcp6" : (tcp6,),
+            "udp"  : (udp4, udp6),
+            "udp4" : (udp4,),
+            "udp6" : (udp6,),
+            "inet" : (tcp4, tcp6, udp4, udp6),
+            "inet4": (tcp4, udp4),
+            "inet6": (tcp6, udp6),
+        }
+        if kind not in tmap:
+            raise ValueError("invalid %r kind argument; choose between %s"
+                             % (kind, ', '.join([repr(x) for x in tmap])))
+        ret = []
+        for f, family, type_ in tmap[kind]:
+            ret += process("/proc/net/%s" % f, family, type_)
+        return ret
 
-        tcp4 = process("/proc/net/tcp", socket.AF_INET, socket.SOCK_STREAM)
-        tcp6 = process("/proc/net/tcp6", socket.AF_INET6, socket.SOCK_STREAM)
-        udp4 = process("/proc/net/udp", socket.AF_INET, socket.SOCK_DGRAM)
-        udp6 = process("/proc/net/udp6", socket.AF_INET6, socket.SOCK_DGRAM)
-        return tcp4 + tcp6 + udp4 + udp6
 
 #    --- lsof implementation
 #
@@ -514,36 +701,39 @@ class LinuxProcess(object):
 
     @wrap_exceptions
     def get_process_ppid(self):
-        if self.pid == 0:
-            return 0
         f = open("/proc/%s/status" % self.pid)
-        for line in f:
-            if line.startswith("PPid:"):
-                # PPid: nnnn
-                f.close()
-                return int(line.split()[1])
+        try:
+            for line in f:
+                if line.startswith("PPid:"):
+                    # PPid: nnnn
+                    return int(line.split()[1])
+            raise RuntimeError("line not found")
+        finally:
+            f.close()
 
     @wrap_exceptions
     def get_process_uids(self):
-        if self.pid == 0:
-            return ntuple_uids(0, 0, 0)
         f = open("/proc/%s/status" % self.pid)
-        for line in f:
-            if line.startswith('Uid:'):
-                f.close()
-                _, real, effective, saved, fs = line.split()
-                return ntuple_uids(int(real), int(effective), int(saved))
+        try:
+            for line in f:
+                if line.startswith('Uid:'):
+                    _, real, effective, saved, fs = line.split()
+                    return ntuple_uids(int(real), int(effective), int(saved))
+            raise RuntimeError("line not found")
+        finally:
+            f.close()
 
     @wrap_exceptions
     def get_process_gids(self):
-        if self.pid == 0:
-            return ntuple_uids(0, 0, 0)
         f = open("/proc/%s/status" % self.pid)
-        for line in f:
-            if line.startswith('Gid:'):
-                f.close()
-                _, real, effective, saved, fs = line.split()
-                return ntuple_gids(int(real), int(effective), int(saved))
+        try:
+            for line in f:
+                if line.startswith('Gid:'):
+                    _, real, effective, saved, fs = line.split()
+                    return ntuple_gids(int(real), int(effective), int(saved))
+            raise RuntimeError("line not found")
+        finally:
+            f.close()
 
     @staticmethod
     def _decode_address(addr, family):
@@ -553,9 +743,9 @@ class LinuxProcess(object):
         "0500000A:0016" -> ("10.0.0.5", 22)
         "0000000000000000FFFF00000100007F:9E49" -> ("::ffff:127.0.0.1", 40521)
 
-        The IPv4 address portion is a little-endian four-byte hexadecimal
-        number; that is, the least significant byte is listed first,
-        so we need to reverse the order of the bytes to convert it
+        The IP address portion is a little or big endian four-byte
+        hexadecimal number; that is, the least significant byte is listed
+        first, so we need to reverse the order of the bytes to convert it
         to an IP address.
         The port is represented as a two-byte hexadecimal number.
 
@@ -571,16 +761,22 @@ class LinuxProcess(object):
         if not port:
             return ()
         if family == socket.AF_INET:
-            ip = socket.inet_ntop(family, base64.b16decode(ip)[::-1])
+            # see: http://code.google.com/p/psutil/issues/detail?id=201
+            if sys.byteorder == 'little':
+                ip = socket.inet_ntop(family, base64.b16decode(ip)[::-1])
+            else:
+                ip = socket.inet_ntop(family, base64.b16decode(ip))
         else:  # IPv6
             # old version - let's keep it, just in case...
             #ip = ip.decode('hex')
             #return socket.inet_ntop(socket.AF_INET6,
             #          ''.join(ip[i:i+4][::-1] for i in xrange(0, 16, 4)))
             ip = base64.b16decode(ip)
-            ip = socket.inet_ntop(socket.AF_INET6,
+            # see: http://code.google.com/p/psutil/issues/detail?id=201
+            if sys.byteorder == 'little':
+                ip = socket.inet_ntop(socket.AF_INET6,
                                 struct.pack('>4I', *struct.unpack('<4I', ip)))
+            else:
+                ip = socket.inet_ntop(socket.AF_INET6,
+                                struct.pack('<4I', *struct.unpack('<4I', ip)))
         return (ip, port)
-
-PlatformProcess = LinuxProcess
-
