@@ -1,7 +1,12 @@
 #!/usr/bin/env python
 #
-# $Id: _psosx.py 940 2011-02-24 19:19:58Z g.rodola $
+# $Id: _psosx.py 1159 2011-10-14 18:42:54Z g.rodola@gmail.com $
 #
+# Copyright (c) 2009, Jay Loden, Giampaolo Rodola'. All rights reserved.
+# Use of this source code is governed by a BSD-style license that can be
+# found in the LICENSE file.
+
+"""OSX platform implementation."""
 
 import errno
 import os
@@ -13,49 +18,67 @@ from psutil.error import AccessDenied, NoSuchProcess, TimeoutExpired
 from psutil._compat import namedtuple
 from psutil._common import *
 
-__all__ = base_module_namespace[:]
+__extra__all__ = []
 
 # --- constants
 
 NUM_CPUS = _psutil_osx.get_num_cpus()
-TOTAL_PHYMEM = _psutil_osx.get_total_phymem()
 BOOT_TIME = _psutil_osx.get_system_boot_time()
+_TERMINAL_MAP = _psposix._get_terminal_map()
+_cputimes_ntuple = namedtuple('cputimes', 'user nice system idle')
 
 # --- functions
 
-def avail_phymem():
-    "Return the amount of physical memory available on the system, in bytes."
-    return _psutil_osx.get_avail_phymem()
+def phymem_usage():
+    """Physical system memory as a (total, used, free) tuple."""
+    total = _psutil_osx.get_total_phymem()
+    free =  _psutil_osx.get_avail_phymem()
+    used = total - free
+    percent = usage_percent(used, total, _round=1)
+    return ntuple_sysmeminfo(total, used, free, percent)
 
-def used_phymem():
-    "Return the amount of physical memory currently in use on the system, in bytes."
-    return TOTAL_PHYMEM - _psutil_osx.get_avail_phymem()
+def virtmem_usage():
+    """Virtual system memory as a (total, used, free) tuple."""
+    total = _psutil_osx.get_total_virtmem()
+    free =  _psutil_osx.get_avail_virtmem()
+    used = total - free
+    percent = usage_percent(used, total, _round=1)
+    return ntuple_sysmeminfo(total, used, free, percent)
 
-def total_virtmem():
-    "Return the amount of total virtual memory available on the system, in bytes."
-    return _psutil_osx.get_total_virtmem()
-
-def avail_virtmem():
-    "Return the amount of virtual memory currently in use on the system, in bytes."
-    return _psutil_osx.get_avail_virtmem()
-
-def used_virtmem():
-    """Return the amount of used memory currently in use on the system, in bytes."""
-    return _psutil_osx.get_total_virtmem() - _psutil_osx.get_avail_virtmem()
-
-_cputimes_ntuple = namedtuple('cputimes', 'user nice system idle')
 def get_system_cpu_times():
     """Return system CPU times as a namedtuple."""
     user, nice, system, idle = _psutil_osx.get_system_cpu_times()
     return _cputimes_ntuple(user, nice, system, idle)
 
-def get_pid_list():
-    """Returns a list of PIDs currently running on the system."""
-    return _psutil_osx.get_pid_list()
+def get_system_per_cpu_times():
+    """Return system CPU times as a named tuple"""
+    ret = []
+    for cpu_t in _psutil_osx.get_system_per_cpu_times():
+        user, nice, system, idle = cpu_t
+        item = _cputimes_ntuple(user, nice, system, idle)
+        ret.append(item)
+    return ret
 
-def pid_exists(pid):
-    """Check For the existence of a unix pid."""
-    return _psposix.pid_exists(pid)
+def disk_partitions(all=False):
+    retlist = []
+    partitions = _psutil_osx.get_disk_partitions()
+    for partition in partitions:
+        device, mountpoint, fstype = partition
+        if device == 'none':
+            device = ''
+        if not all:
+            if not os.path.isabs(device) \
+            or not os.path.exists(device):
+                continue
+        ntuple = ntuple_partition(device, mountpoint, fstype)
+        retlist.append(ntuple)
+    return retlist
+
+get_pid_list = _psutil_osx.get_pid_list
+pid_exists = _psposix.pid_exists
+get_disk_usage = _psposix.get_disk_usage
+network_io_counters = _psutil_osx.get_network_io_counters
+disk_io_counters = _psutil_osx.get_disk_io_counters
 
 # --- decorator
 
@@ -84,7 +107,7 @@ _status_map = {
     _psutil_osx.SZOMB : STATUS_ZOMBIE,
 }
 
-class OSXProcess(object):
+class Process(object):
     """Wrapper class around underlying C implementation."""
 
     __slots__ = ["pid", "_process_name"]
@@ -122,12 +145,19 @@ class OSXProcess(object):
         real, effective, saved = _psutil_osx.get_process_uids(self.pid)
         return ntuple_uids(real, effective, saved)
 
-
     @wrap_exceptions
     def get_process_gids(self):
         real, effective, saved = _psutil_osx.get_process_gids(self.pid)
         return ntuple_gids(real, effective, saved)
 
+    @wrap_exceptions
+    def get_process_terminal(self):
+        tty_nr = _psutil_osx.get_process_tty_nr(self.pid)
+        try:
+            return _TERMINAL_MAP[tty_nr]
+        except KeyError:
+            return None
+
     @wrap_exceptions
     def get_memory_info(self):
         """Return a tuple with the process' RSS and VMS size."""
@@ -164,11 +194,16 @@ class OSXProcess(object):
         return files
 
     @wrap_exceptions
-    def get_connections(self):
+    def get_connections(self, kind='inet'):
         """Return etwork connections opened by a process as a list of
-        namedtuples."""
-        retlist = _psutil_osx.get_process_connections(self.pid)
-        return [ntuple_connection(*conn) for conn in retlist]
+        namedtuples.
+        """
+        if kind not in conn_tmap:
+            raise ValueError("invalid %r kind argument; choose between %s"
+                             % (kind, ', '.join([repr(x) for x in conn_tmap])))
+        families, types = conn_tmap[kind]
+        ret = _psutil_osx.get_process_connections(self.pid, families, types)
+        return [ntuple_connection(*conn) for conn in ret]
 
     @wrap_exceptions
     def process_wait(self, timeout=None):
@@ -201,8 +236,3 @@ class OSXProcess(object):
             ntuple = ntuple_thread(thread_id, utime, stime)
             retlist.append(ntuple)
         return retlist
-
-
-
-PlatformProcess = OSXProcess
-
