@@ -1,5 +1,9 @@
 /*
- * $Id: _psutil_linux.c 892 2011-01-13 21:42:53Z g.rodola $
+ * $Id: _psutil_linux.c 1166 2011-10-17 22:18:07Z g.rodola $
+ *
+ * Copyright (c) 2009, Jay Loden, Giampaolo Rodola'. All rights reserved.
+ * Use of this source code is governed by a BSD-style license that can be
+ * found in the LICENSE file.
  *
  * Linux-specific functions.
  */
@@ -7,7 +11,9 @@
 #include <Python.h>
 #include <errno.h>
 #include <stdlib.h>
+#include <mntent.h>
 #include <sys/syscall.h>
+#include <sys/sysinfo.h>
 #include <linux/unistd.h>
 
 #include "_psutil_linux.h"
@@ -17,22 +23,22 @@
 
 #if HAS_IOPRIO
 enum {
-	IOPRIO_WHO_PROCESS = 1,
+    IOPRIO_WHO_PROCESS = 1,
 };
 
 static inline int
 ioprio_get(int which, int who)
 {
-	return syscall(__NR_ioprio_get, which, who);
+    return syscall(__NR_ioprio_get, which, who);
 }
 
 static inline int
 ioprio_set(int which, int who, int ioprio)
 {
-	return syscall(__NR_ioprio_set, which, who, ioprio);
+    return syscall(__NR_ioprio_set, which, who, ioprio);
 }
 
-#define IOPRIO_CLASS_SHIFT	13
+#define IOPRIO_CLASS_SHIFT  13
 #define IOPRIO_PRIO_MASK  ((1UL << IOPRIO_CLASS_SHIFT) - 1)
 
 #define IOPRIO_PRIO_CLASS(mask)  ((mask) >> IOPRIO_CLASS_SHIFT)
@@ -47,7 +53,7 @@ static PyObject*
 linux_ioprio_get(PyObject* self, PyObject* args)
 {
     long pid;
-	int ioprio, ioclass, iodata;
+    int ioprio, ioclass, iodata;
     if (! PyArg_ParseTuple(args, "l", &pid)) {
         return NULL;
     }
@@ -55,8 +61,8 @@ linux_ioprio_get(PyObject* self, PyObject* args)
     if (ioprio == -1) {
         return PyErr_SetFromErrno(PyExc_OSError);
     }
-	ioclass = IOPRIO_PRIO_CLASS(ioprio);
-	iodata = IOPRIO_PRIO_DATA(ioprio);
+    ioclass = IOPRIO_PRIO_CLASS(ioprio);
+    iodata = IOPRIO_PRIO_DATA(ioprio);
     return Py_BuildValue("ii", ioclass, iodata);
 }
 
@@ -77,8 +83,8 @@ linux_ioprio_set(PyObject* self, PyObject* args)
         return NULL;
     }
     ioprio = IOPRIO_PRIO_VALUE(ioclass, iodata);
-	retval = ioprio_set(IOPRIO_WHO_PROCESS, pid, ioprio);
-	if (retval == -1) {
+    retval = ioprio_set(IOPRIO_WHO_PROCESS, pid, ioprio);
+    if (retval == -1) {
         return PyErr_SetFromErrno(PyExc_OSError);
     }
     Py_INCREF(Py_None);
@@ -86,6 +92,58 @@ linux_ioprio_set(PyObject* self, PyObject* args)
 }
 #endif
 
+
+/*
+ * Return disk mounted partitions as a list of tuples including device,
+ * mount point and filesystem type
+ */
+static PyObject*
+get_disk_partitions(PyObject* self, PyObject* args)
+{
+    FILE *file;
+    struct mntent *entry;
+    PyObject* py_retlist = PyList_New(0);
+    PyObject* py_tuple;
+
+    // MOUNTED constant comes from mntent.h and it's == '/etc/mtab'
+    Py_BEGIN_ALLOW_THREADS
+    file = setmntent(MOUNTED, "r");
+    Py_END_ALLOW_THREADS
+    if ((file == 0) || (file == NULL)) {
+        return PyErr_SetFromErrno(PyExc_OSError);
+    }
+
+    while ((entry = getmntent(file))) {
+        if (entry == NULL) {
+            endmntent(file);
+            return PyErr_Format(PyExc_RuntimeError, "getmntent() failed");
+        }
+        py_tuple = Py_BuildValue("(sss)", entry->mnt_fsname,  // device
+                                          entry->mnt_dir,     // mount point
+                                          entry->mnt_type);   // fs type
+        PyList_Append(py_retlist, py_tuple);
+        Py_XDECREF(py_tuple);
+    }
+
+    endmntent(file);
+    return py_retlist;
+}
+
+
+/*
+ * Return physical memory usage as a (total, free, buffer) tuple.
+ */
+static PyObject*
+get_physmem(PyObject* self, PyObject* args)
+{
+    struct sysinfo info;
+    if (sysinfo(&info) != 0) {
+        return PyErr_SetFromErrno(PyExc_OSError);
+    }
+    return Py_BuildValue("(lll)", info.totalram, info.freeram, info.bufferram);
+}
+
+
 /*
  * Define the psutil C module methods and initialize the module.
  */
@@ -98,6 +156,12 @@ PsutilMethods[] =
      {"ioprio_set", linux_ioprio_set, METH_VARARGS,
         "Set process I/O priority"},
 #endif
+     {"get_disk_partitions", get_disk_partitions, METH_VARARGS,
+        "Return disk mounted partitions as a list of tuples including "
+        "device, mount point and filesystem type"},
+     {"get_physmem", get_physmem, METH_VARARGS,
+        "The physical memory usage as a (total, free, buffer) tuple."},
+
      {NULL, NULL, 0, NULL}
 };
 
@@ -109,7 +173,6 @@ struct module_state {
 #define GETSTATE(m) ((struct module_state*)PyModule_GetState(m))
 #else
 #define GETSTATE(m) (&_state)
-static struct module_state _state;
 #endif
 
 #if PY_MAJOR_VERSION >= 3
@@ -158,7 +221,6 @@ void init_psutil_linux(void)
     if (module == NULL) {
         INITERROR;
     }
-    struct module_state *st = GETSTATE(module);
 #if PY_MAJOR_VERSION >= 3
     return module;
 #endif
