@@ -1,5 +1,9 @@
 /*
- * $Id: _psutil_osx.c 940 2011-02-24 19:19:58Z g.rodola $
+ * $Id: _psutil_osx.c 1193 2011-10-22 18:24:53Z g.rodola@gmail.com $
+ *
+ * Copyright (c) 2009, Jay Loden, Giampaolo Rodola'. All rights reserved.
+ * Use of this source code is governed by a BSD-style license that can be
+ * found in the LICENSE file.
  *
  * OS X platform-specific module methods for _psutil_osx
  */
@@ -16,6 +20,7 @@
 #include <sys/proc_info.h>
 #include <netinet/tcp_fsm.h>
 #include <arpa/inet.h>
+#include <net/if_dl.h>
 
 #include <mach/mach.h>
 #include <mach/task.h>
@@ -25,6 +30,12 @@
 #include <mach/mach_traps.h>
 #include <mach/shared_memory_server.h>
 
+#include <CoreFoundation/CoreFoundation.h>
+#include <IOKit/IOKitLib.h>
+#include <IOKit/storage/IOBlockStorageDriver.h>
+#include <IOKit/storage/IOMedia.h>
+#include <IOKit/IOBSD.h>
+
 #include "_psutil_osx.h"
 #include "_psutil_common.h"
 #include "arch/osx/process_info.h"
@@ -58,8 +69,8 @@ get_pid_list(PyObject* self, PyObject* args)
             Py_XDECREF(pid);
             proclist++;
         }
+        free(orig_address);
     }
-    free(orig_address);
     return retlist;
 }
 
@@ -159,6 +170,24 @@ get_process_gids(PyObject* self, PyObject* args)
 }
 
 
+/*
+ * Return process controlling terminal number as an integer.
+ */
+static PyObject*
+get_process_tty_nr(PyObject* self, PyObject* args)
+{
+    long pid;
+    struct kinfo_proc kp;
+    if (! PyArg_ParseTuple(args, "l", &pid)) {
+        return NULL;
+    }
+    if (get_kinfo_proc(pid, &kp) == -1) {
+        return NULL;
+    }
+    return Py_BuildValue("i", kp.kp_eproc.e_tdev);
+}
+
+
 /*
  * Return a Python integer indicating the number of CPUs on the system.
  */
@@ -351,7 +380,7 @@ static PyObject*
 get_process_num_threads(PyObject* self, PyObject* args)
 {
     long pid;
-    int err;
+    int err, ret;
     unsigned int info_count = TASK_BASIC_INFO_COUNT;
     mach_port_t task;
     struct task_basic_info tasks_info;
@@ -382,14 +411,19 @@ get_process_num_threads(PyObject* self, PyObject* args)
                 return PyErr_Format(PyExc_RuntimeError,
                                     "task_info(TASK_BASIC_INFO) failed");
         }
-
         err = task_threads(task, &thread_list, &thread_count);
         if (err == KERN_SUCCESS) {
+            ret = vm_deallocate(task, (vm_address_t)thread_list,
+                                thread_count * sizeof(int));
+            if (ret != KERN_SUCCESS) {
+                printf("vm_deallocate() failed\n");
+            }
             return Py_BuildValue("l", (long)thread_count);
         }
+        else {
+            return PyErr_Format(PyExc_RuntimeError, "task_thread() failed");
+        }
     }
-
-
     else {
         if (! pid_exists(pid) ) {
             return NoSuchProcess();
@@ -523,6 +557,50 @@ get_system_cpu_times(PyObject* self, PyObject* args)
 }
 
 
+/*
+ * Return a Python list of tuple representing per-cpu times
+ */
+static PyObject*
+get_system_per_cpu_times(PyObject* self, PyObject* args)
+{
+    natural_t cpu_count;
+    processor_info_array_t info_array;
+    mach_msg_type_number_t info_count;
+    kern_return_t error;
+    processor_cpu_load_info_data_t* cpu_load_info;
+    PyObject* py_retlist = PyList_New(0);
+    PyObject* py_cputime;
+    int i, ret;
+
+    error = host_processor_info(mach_host_self(), PROCESSOR_CPU_LOAD_INFO,
+                                &cpu_count, &info_array, &info_count);
+    if (error != KERN_SUCCESS) {
+        return PyErr_Format(PyExc_RuntimeError,
+              "Error in host_processor_info(): %s", mach_error_string(error));
+    }
+
+    cpu_load_info = (processor_cpu_load_info_data_t*) info_array;
+
+    for (i = 0; i < cpu_count; i++) {
+        py_cputime = Py_BuildValue("(dddd)",
+               (double)cpu_load_info[i].cpu_ticks[CPU_STATE_USER] / CLK_TCK,
+               (double)cpu_load_info[i].cpu_ticks[CPU_STATE_NICE] / CLK_TCK,
+               (double)cpu_load_info[i].cpu_ticks[CPU_STATE_SYSTEM] / CLK_TCK,
+               (double)cpu_load_info[i].cpu_ticks[CPU_STATE_IDLE] / CLK_TCK
+              );
+        PyList_Append(py_retlist, py_cputime);
+        Py_XDECREF(py_cputime);
+    }
+
+    ret = vm_deallocate(mach_task_self(), (vm_address_t)info_array,
+                        info_count * sizeof(int));
+    if (ret != KERN_SUCCESS) {
+        printf("vm_deallocate() failed\n");
+    }
+    return py_retlist;
+}
+
+
 /*
  * Return a Python float indicating the system boot time expressed in
  * seconds since the epoch.
@@ -545,6 +623,54 @@ get_system_boot_time(PyObject* self, PyObject* args)
 }
 
 
+/*
+ * Return a list of tuples including device, mount point and fs type
+ * for all partitions mounted on the system.
+ */
+static PyObject*
+get_disk_partitions(PyObject* self, PyObject* args)
+{
+    int num;
+    int i;
+    long len;
+    struct statfs *fs;
+    PyObject* py_retlist = PyList_New(0);
+    PyObject* py_tuple;
+
+    // get the number of mount points
+    Py_BEGIN_ALLOW_THREADS
+    num = getfsstat(NULL, 0, MNT_NOWAIT);
+    Py_END_ALLOW_THREADS
+    if (num == -1) {
+        PyErr_SetFromErrno(0);
+        return NULL;
+    }
+
+    len = sizeof(*fs) * num;
+    fs = malloc(len);
+
+    Py_BEGIN_ALLOW_THREADS
+    num = getfsstat(fs, len, MNT_NOWAIT);
+    Py_END_ALLOW_THREADS
+    if (num == -1) {
+        free(fs);
+        PyErr_SetFromErrno(0);
+        return NULL;
+    }
+
+    for (i = 0; i < num; i++) {
+        py_tuple = Py_BuildValue("(sss)", fs[i].f_mntfromname,  // device
+                                          fs[i].f_mntonname,    // mount point
+                                          fs[i].f_fstypename);  // fs type
+        PyList_Append(py_retlist, py_tuple);
+        Py_XDECREF(py_tuple);
+    }
+
+    free(fs);
+    return py_retlist;
+}
+
+
 /*
  * Return process status as a Python integer.
  */
@@ -570,7 +696,7 @@ static PyObject*
 get_process_threads(PyObject* self, PyObject* args)
 {
     long pid;
-    int err, j;
+    int err, j, ret;
     kern_return_t kr;
     unsigned int info_count = TASK_BASIC_INFO_COUNT;
     mach_port_t task;
@@ -632,6 +758,12 @@ get_process_threads(PyObject* self, PyObject* args)
         Py_XDECREF(pyTuple);
     }
 
+    ret = vm_deallocate(task, (vm_address_t)thread_list,
+                        thread_count * sizeof(int));
+    if (ret != KERN_SUCCESS) {
+        printf("vm_deallocate() failed\n");
+    }
+
     return retList;
 }
 
@@ -786,16 +918,26 @@ get_process_connections(PyObject* self, PyObject* args)
     struct proc_fdinfo *fdp_pointer;
     struct socket_fdinfo si;
 
-
     PyObject *retList = PyList_New(0);
     PyObject *tuple = NULL;
     PyObject *laddr = NULL;
     PyObject *raddr = NULL;
+    PyObject *af_filter = NULL;
+    PyObject *type_filter = NULL;
 
-    if (! PyArg_ParseTuple(args, "l", &pid)) {
+    if (! PyArg_ParseTuple(args, "lOO", &pid, &af_filter, &type_filter)) {
         return NULL;
     }
 
+    if (!PySequence_Check(af_filter) || !PySequence_Check(type_filter)) {
+        PyErr_SetString(PyExc_TypeError, "arg 2 or 3 is not a sequence");
+        return NULL;
+    }
+
+    if (pid == 0) {
+        return retList;
+    }
+
     pidinfo_result = proc_pidinfo(pid, PROC_PIDLISTFDS, 0, NULL, 0);
     if (pidinfo_result <= 0) {
         goto error;
@@ -846,15 +988,12 @@ get_process_connections(PyObject* self, PyObject* args)
             int fd, family, type, lport, rport;
             char lip[200], rip[200];
             char *state;
+            int inseq;
 
             fd = (int)fdp_pointer->proc_fd;
             family = si.psi.soi_family;
             type = si.psi.soi_kind;
 
-            if ((family != AF_INET) && (family != AF_INET6)) {
-                continue;
-            }
-
             if (type == 2)
                 type = SOCK_STREAM;
             else if (type == 1)
@@ -862,12 +1001,18 @@ get_process_connections(PyObject* self, PyObject* args)
             else
                 continue;
 
+            // apply filters
+            inseq = PySequence_Contains(af_filter, PyLong_FromLong((long)family));
+            if (inseq == 0)
+                continue;
+            inseq = PySequence_Contains(type_filter, PyLong_FromLong((long)type));
+            if (inseq == 0)
+                continue;
+
             if (errno != 0) {
-                printf("errno 1 = %i\n", errno);
                 return PyErr_SetFromErrno(PyExc_OSError);
             }
 
-
             if (family == AF_INET) {
                 inet_ntop(AF_INET,
                           &si.psi.soi_proto.pri_tcp.tcpsi_ini.insi_laddr.ina_46.i46a_addr4,
@@ -927,6 +1072,232 @@ error:
 }
 
 
+/*
+ * Return a Python list of named tuples with overall network I/O information
+ */
+static PyObject*
+get_network_io_counters(PyObject* self, PyObject* args)
+{
+    PyObject* py_retdict = PyDict_New();
+    PyObject* py_ifc_info;
+
+    char *buf = NULL, *lim, *next;
+    struct if_msghdr *ifm;
+    int mib[6];
+    size_t len;
+
+    mib[0] = CTL_NET;          // networking subsystem
+    mib[1] = PF_ROUTE;         // type of information
+    mib[2] = 0;                // protocol (IPPROTO_xxx)
+    mib[3] = 0;                // address family
+    mib[4] = NET_RT_IFLIST2;   // operation
+    mib[5] = 0;
+
+    if (sysctl(mib, 6, NULL, &len, NULL, 0) < 0) {
+        Py_DECREF(py_retdict);
+        PyErr_SetFromErrno(0);
+        return NULL;
+    }
+
+    buf = malloc(len);
+
+    if (sysctl(mib, 6, buf, &len, NULL, 0) < 0) {
+        if (buf) {
+            free(buf);
+        }
+        Py_DECREF(py_retdict);
+        PyErr_SetFromErrno(0);
+        return NULL;
+    }
+
+    lim = buf + len;
+
+    for (next = buf; next < lim; ) {
+        ifm = (struct if_msghdr *)next;
+        next += ifm->ifm_msglen;
+
+        if (ifm->ifm_type == RTM_IFINFO2) {
+            struct if_msghdr2 *if2m = (struct if_msghdr2 *)ifm;
+            struct sockaddr_dl *sdl = (struct sockaddr_dl *)(if2m + 1);
+            char ifc_name[32];
+
+            strncpy(ifc_name, sdl->sdl_data, sdl->sdl_nlen);
+            ifc_name[sdl->sdl_nlen] = 0;
+
+            py_ifc_info = Py_BuildValue("(KKKK)",
+                                        if2m->ifm_data.ifi_obytes,
+                                        if2m->ifm_data.ifi_ibytes,
+                                        if2m->ifm_data.ifi_opackets,
+                                        if2m->ifm_data.ifi_ipackets);
+            PyDict_SetItemString(py_retdict, ifc_name, py_ifc_info);
+            Py_XDECREF(py_ifc_info);
+        }
+        else {
+            continue;
+        }
+    }
+
+    free(buf);
+
+    return py_retdict;
+}
+
+
+/*
+ * Return a Python dict of tuples for disk I/O information
+ */
+static PyObject*
+get_disk_io_counters(PyObject* self, PyObject* args)
+{
+    PyObject* py_retdict = PyDict_New();
+    PyObject* py_disk_info;
+
+    CFDictionaryRef parent_dict;
+    CFDictionaryRef props_dict;
+    CFDictionaryRef stats_dict;
+    io_registry_entry_t parent;
+    io_registry_entry_t disk;
+    io_iterator_t disk_list;
+
+    /* Get list of disks */
+    if (IOServiceGetMatchingServices(kIOMasterPortDefault,
+                                     IOServiceMatching(kIOMediaClass),
+                                     &disk_list) != kIOReturnSuccess) {
+        Py_DECREF(py_retdict);
+        PyErr_SetString(PyExc_RuntimeError, "Unable to get the list of disks.");
+        return NULL;
+    }
+
+    /* Iterate over disks */
+    while ((disk = IOIteratorNext(disk_list)) != 0) {
+        parent_dict = NULL;
+        props_dict = NULL;
+        stats_dict = NULL;
+
+        if (IORegistryEntryGetParentEntry(disk, kIOServicePlane, &parent) != kIOReturnSuccess) {
+            PyErr_SetString(PyExc_RuntimeError, "Unable to get the disk's parent.");
+            Py_DECREF(py_retdict);
+            IOObjectRelease(disk);
+            return NULL;
+        }
+
+        if (IOObjectConformsTo(parent, "IOBlockStorageDriver")) {
+            if(IORegistryEntryCreateCFProperties(
+                                     disk,
+                                     (CFMutableDictionaryRef *) &parent_dict,
+                                     kCFAllocatorDefault,
+                                     kNilOptions) != kIOReturnSuccess)
+            {
+                PyErr_SetString(PyExc_RuntimeError,
+                                "Unable to get the parent's properties.");
+                Py_DECREF(py_retdict);
+                IOObjectRelease(disk);
+                IOObjectRelease(parent);
+                return NULL;
+            }
+
+            if (IORegistryEntryCreateCFProperties(parent,
+                                          (CFMutableDictionaryRef *) &props_dict,
+                                          kCFAllocatorDefault,
+                                          kNilOptions) != kIOReturnSuccess)
+            {
+                PyErr_SetString(PyExc_RuntimeError,
+                                "Unable to get the disk properties.");
+                Py_DECREF(py_retdict);
+                IOObjectRelease(disk);
+                return NULL;
+            }
+
+            const int kMaxDiskNameSize = 64;
+            CFStringRef disk_name_ref = (CFStringRef)CFDictionaryGetValue(
+                                                    parent_dict,
+                                                    CFSTR(kIOBSDNameKey));
+            char disk_name[kMaxDiskNameSize];
+
+            CFStringGetCString(disk_name_ref,
+                               disk_name,
+                               kMaxDiskNameSize,
+                               CFStringGetSystemEncoding());
+
+            stats_dict = (CFDictionaryRef)CFDictionaryGetValue(
+                                    props_dict,
+                                    CFSTR(kIOBlockStorageDriverStatisticsKey));
+
+            if (stats_dict == NULL) {
+                PyErr_SetString(PyExc_RuntimeError, "Unable to get disk stats.");
+                Py_DECREF(py_retdict);
+                CFRelease(props_dict);
+                IOObjectRelease(disk);
+                IOObjectRelease(parent);
+                return NULL;
+            }
+
+            CFNumberRef number;
+            int64_t reads, writes, read_bytes, write_bytes, read_time, write_time = 0;
+
+            /* Get disk reads/writes */
+            if ((number = (CFNumberRef)CFDictionaryGetValue(
+                            stats_dict,
+                            CFSTR(kIOBlockStorageDriverStatisticsReadsKey))))
+            {
+                CFNumberGetValue(number, kCFNumberSInt64Type, &reads);
+            }
+            if ((number = (CFNumberRef)CFDictionaryGetValue(
+                            stats_dict,
+                            CFSTR(kIOBlockStorageDriverStatisticsWritesKey))))
+            {
+                CFNumberGetValue(number, kCFNumberSInt64Type, &writes);
+            }
+
+            /* Get disk bytes read/written */
+            if ((number = (CFNumberRef)CFDictionaryGetValue(
+                        stats_dict,
+                        CFSTR(kIOBlockStorageDriverStatisticsBytesReadKey))))
+            {
+                CFNumberGetValue(number, kCFNumberSInt64Type, &read_bytes);
+            }
+            if ((number = (CFNumberRef)CFDictionaryGetValue(
+                stats_dict,
+                CFSTR(kIOBlockStorageDriverStatisticsBytesWrittenKey))))
+            {
+                CFNumberGetValue(number, kCFNumberSInt64Type, &write_bytes);
+            }
+
+            /* Get disk time spent reading/writing (nanoseconds) */
+            if ((number = (CFNumberRef)CFDictionaryGetValue(
+                    stats_dict,
+                    CFSTR(kIOBlockStorageDriverStatisticsTotalReadTimeKey))))
+            {
+                CFNumberGetValue(number, kCFNumberSInt64Type, &read_time);
+            }
+            if ((number = (CFNumberRef)CFDictionaryGetValue(
+                    stats_dict,
+                    CFSTR(kIOBlockStorageDriverStatisticsTotalWriteTimeKey)))) {
+                CFNumberGetValue(number, kCFNumberSInt64Type, &write_time);
+            }
+
+            // Read/Write time on OS X comes back in nanoseconds and in psutil
+            // we've standardized on milliseconds so do the conversion.
+            py_disk_info = Py_BuildValue("(KKKKKK)",
+                                         reads, writes,
+                                         read_bytes, write_bytes,
+                                         read_time / 1000, write_time / 1000);
+            PyDict_SetItemString(py_retdict, disk_name, py_disk_info);
+            Py_XDECREF(py_disk_info);
+
+            CFRelease(parent_dict);
+            IOObjectRelease(parent);
+            CFRelease(props_dict);
+            IOObjectRelease(disk);
+        }
+    }
+
+    IOObjectRelease (disk_list);
+
+    return py_retdict;
+}
+
+
 /*
  * define the psutil C module methods and initialize the module.
  */
@@ -962,7 +1333,8 @@ PsutilMethods[] =
          "Return files opened by process as a list of tuples"},
      {"get_process_connections", get_process_connections, METH_VARARGS,
          "Get process TCP and UDP connections as a list of tuples"},
-
+     {"get_process_tty_nr", get_process_tty_nr, METH_VARARGS,
+         "Return process tty number as an integer"},
 
      // --- system-related functions
 
@@ -980,9 +1352,18 @@ PsutilMethods[] =
          "Return the amount of available virtual memory, in bytes"},
      {"get_system_cpu_times", get_system_cpu_times, METH_VARARGS,
          "Return system cpu times as a tuple (user, system, nice, idle, irc)"},
+     {"get_system_per_cpu_times", get_system_per_cpu_times, METH_VARARGS,
+         "Return system per-cpu times as a list of tuples"},
      {"get_system_boot_time", get_system_boot_time, METH_VARARGS,
          "Return a float indicating the system boot time expressed in "
          "seconds since the epoch"},
+     {"get_disk_partitions", get_disk_partitions, METH_VARARGS,
+         "Return a list of tuples including device, mount point and "
+         "fs type for all partitions mounted on the system."},
+     {"get_network_io_counters", get_network_io_counters, METH_VARARGS,
+         "Return dict of tuples of networks I/O information."},
+     {"get_disk_io_counters", get_disk_io_counters, METH_VARARGS,
+         "Return dict of tuples of disks I/O information."},
 
      {NULL, NULL, 0, NULL}
 };
@@ -996,7 +1377,6 @@ struct module_state {
 #define GETSTATE(m) ((struct module_state*)PyModule_GetState(m))
 #else
 #define GETSTATE(m) (&_state)
-static struct module_state _state;
 #endif
 
 #if PY_MAJOR_VERSION >= 3
@@ -1055,16 +1435,7 @@ init_psutil_osx(void)
     if (module == NULL) {
         INITERROR;
     }
-    struct module_state *st = GETSTATE(module);
-
-    st->error = PyErr_NewException("_psutil_osx.Error", NULL, NULL);
-    if (st->error == NULL) {
-        Py_DECREF(module);
-        INITERROR;
-    }
 #if PY_MAJOR_VERSION >= 3
     return module;
 #endif
 }
-
-
