@@ -1,5 +1,9 @@
 /*
- * $Id: security.c 435 2009-09-19 09:32:29Z jloden $
+ * $Id: security.c 1142 2011-10-05 18:45:49Z g.rodola $
+ *
+ * Copyright (c) 2009, Jay Loden, Giampaolo Rodola'. All rights reserved.
+ * Use of this source code is governed by a BSD-style license that can be
+ * found in the LICENSE file.
  *
  * Security related functions for Windows platform (Set privileges such as
  * SeDebug), as well as security helper functions.
@@ -11,7 +15,8 @@
 /*
  * Convert a process handle to a process token handle.
  */
-HANDLE token_from_handle(HANDLE hProcess) {
+HANDLE
+token_from_handle(HANDLE hProcess) {
     HANDLE hToken = NULL;
 
     if (! OpenProcessToken(hProcess, TOKEN_QUERY, &hToken) ) {
@@ -25,7 +30,7 @@ HANDLE token_from_handle(HANDLE hProcess) {
 /*
  * http://www.ddj.com/windows/184405986
  *
- * theres a way to determine whether were running under the Local System
+ * There's a way to determine whether we're running under the Local System
  * account. However (you guessed it), we have to call more Win32 functions to
  * determine this. Backing up through the code listing, we need to make another
  * call to GetTokenInformation, but instead of passing through the TOKEN_USER
@@ -170,7 +175,7 @@ int SetSeDebug()
                          ){
         if (GetLastError() == ERROR_NO_TOKEN){
             if (!ImpersonateSelf(SecurityImpersonation)){
-                //Log2File("Error setting impersonation [SetSeDebug()]", L_DEBUG);
+                CloseHandle(hToken);
                 return 0;
             }
             if (!OpenThreadToken(GetCurrentThread(),
@@ -178,7 +183,8 @@ int SetSeDebug()
                                  FALSE,
                                  &hToken)
                                  ){
-                //Log2File("Error Opening Thread Token", L_DEBUG);
+                RevertToSelf();
+                CloseHandle(hToken);
                 return 0;
             }
         }
@@ -186,10 +192,12 @@ int SetSeDebug()
 
     // enable SeDebugPrivilege (open any process)
     if (! SetPrivilege(hToken, SE_DEBUG_NAME, TRUE)){
-        //Log2File("Error setting SeDebug Privilege [SetPrivilege()]", L_WARN);
+        RevertToSelf();
+        CloseHandle(hToken);
         return 0;
     }
 
+    RevertToSelf();
     CloseHandle(hToken);
     return 1;
 }
